#include "scannerControl.h"
#include "webpage.h"

httpd_handle_t camera_httpd = NULL;

char value[32];
char resp[32];

static esp_err_t index_handler(httpd_req_t *req){
    httpd_resp_set_type(req, "text/html");
    httpd_resp_set_hdr(req, "Content-Encoding", "gzip");
    sensor_t * s = esp_camera_sensor_get();
    return httpd_resp_send(req, (const char *)index_html_gz, index_html_gz_len);
}

static esp_err_t shoot_handler(httpd_req_t *req) {
  char*  buf;
  size_t buf_len;
  int led_on = 0;

  if (status != STATUS_IDLE) {
    sprintf(resp, "BUSY");
    return httpd_resp_send(req, resp, strlen(resp));
  }

  status = STATUS_SHOOTING;

  buf_len = httpd_req_get_url_query_len(req) + 1;
  if (buf_len > 1) {
    buf = (char*)malloc(buf_len);
    if (!buf) {
      httpd_resp_send_500(req);
      return ESP_FAIL;
    }
    if (httpd_req_get_url_query_str(req, buf, buf_len) == ESP_OK) {
      if (httpd_query_key_value(buf, "led_on", value, sizeof(value)) == ESP_OK) {
        led_on = atoi(value);
      }
    } else {
      free(buf);
      httpd_resp_send_404(req);
      return ESP_FAIL;
    }
    free(buf);
  }

  #ifndef CAM_DEBUG
    nodes.setEnabled(2, led_on > 0);
  #endif

  camera_fb_t* fb = NULL;
  esp_err_t res = ESP_OK;
  int64_t fr_start = esp_timer_get_time();

  fb = esp_camera_fb_get();
  if (!fb) {
    httpd_resp_send_500(req);
    status = STATUS_IDLE;
    return ESP_FAIL;
  }

  #ifndef CAM_DEBUG
    nodes.setEnabled(2, false);
  #endif

  #ifdef CAM_RAW
    httpd_resp_set_type(req, "application/x-binary");
    httpd_resp_set_hdr(req, "Content-Disposition", "attachment; filename=capture.bin");
    httpd_resp_set_hdr(req, "Access-Control-Allow-Origin", "*");
  #else
    httpd_resp_set_type(req, "image/jpeg");
    httpd_resp_set_hdr(req, "Content-Disposition", "inline; filename=capture.jpg");
    httpd_resp_set_hdr(req, "Access-Control-Allow-Origin", "*");
  #endif

  esp_err_t resp = httpd_resp_send(req, (const char *)fb->buf, fb->len);

  status = STATUS_IDLE;

  return resp;
}

static esp_err_t status_handler(httpd_req_t *req) {
  httpd_resp_set_hdr(req, "Access-Control-Allow-Origin", "*");
  if (status == STATUS_IDLE) {
    sprintf(resp, "IDLE");
  } else {
    sprintf(resp, "BUSY");
  }
  return httpd_resp_send(req, resp, strlen(resp));
}

static esp_err_t homing_handler(httpd_req_t *req) {
  httpd_resp_set_hdr(req, "Access-Control-Allow-Origin", "*");
  if (status != STATUS_IDLE) {
    sprintf(resp, "BUSY");
    return httpd_resp_send(req, resp, strlen(resp));
  }

  status = STATUS_HOMING;
  sprintf(resp, "OK");  
  return httpd_resp_send(req, resp, strlen(resp));
}

static esp_err_t move_handler(httpd_req_t *req) {
  char*  buf;
  size_t buf_len;

  float angle;
  int steps;

  httpd_resp_set_hdr(req, "Access-Control-Allow-Origin", "*");

  if (status != STATUS_IDLE) {
    sprintf(resp, "BUSY");
    return httpd_resp_send(req, resp, strlen(resp));
  }

  buf_len = httpd_req_get_url_query_len(req) + 1;
  if (buf_len > 1) {
    buf = (char*)malloc(buf_len);
    if (!buf) {
      httpd_resp_send_500(req);
      return ESP_FAIL;
    }
    if (httpd_req_get_url_query_str(req, buf, buf_len) == ESP_OK) {
      if (httpd_query_key_value(buf, "az", value, sizeof(value)) == ESP_OK) {
        angle = atof(value);
        steps = angle*STEPS_PER_AZ;
        nodes.setTarget(1, steps);
      }
      if (httpd_query_key_value(buf, "el", value, sizeof(value)) == ESP_OK) {
        angle = atof(value);
        angle = angle > MAX_EL ? MAX_EL : angle;
        angle = angle < MIN_EL ? MIN_EL : angle;
        steps = (angle-MIN_EL)*STEPS_PER_EL;
        nodes.setTarget(0, steps);
      }
    } else {
      free(buf);
      httpd_resp_send_404(req);
      return ESP_FAIL;
    }
    free(buf);
  } else {
    httpd_resp_send_404(req);
    return ESP_FAIL;
  }

  status = STATUS_MOVE;
  sprintf(resp, "OK");
  return httpd_resp_send(req, resp, strlen(resp));
}


static esp_err_t set_handler(httpd_req_t *req) {
  char*  buf;
  size_t buf_len;

  int val;

  httpd_resp_set_hdr(req, "Access-Control-Allow-Origin", "*");

  if (status != STATUS_IDLE) {
    sprintf(resp, "BUSY");
    return httpd_resp_send(req, resp, strlen(resp));
  }

  buf_len = httpd_req_get_url_query_len(req) + 1;
  if (buf_len > 1) {
    buf = (char*)malloc(buf_len);
    if (!buf) {
      httpd_resp_send_500(req);
      return ESP_FAIL;
    }
    if (httpd_req_get_url_query_str(req, buf, buf_len) == ESP_OK) {
      if (httpd_query_key_value(buf, "led_val", value, sizeof(value)) == ESP_OK) {
        val = atoi(value);
        nodes.setTarget(2, val);
        status = STATUS_MOVE;
      }
      if (httpd_query_key_value(buf, "led_on", value, sizeof(value)) == ESP_OK) {
        val = atoi(value);
        #ifndef CAM_DEBUG
          if (val) {
            nodes.setEnabled(2, true);
          } else {
            nodes.setEnabled(2, false);
          }
        #endif
      }
      if (httpd_query_key_value(buf, "motor_on", value, sizeof(value)) == ESP_OK) {
        val = atoi(value);
        #ifndef CAM_DEBUG
          if (val) {
            nodes.setEnabled(0, true);
            nodes.setEnabled(1, true);
          } else {
            nodes.setEnabled(0, false);
            nodes.setEnabled(1, false);
          }
        #endif
      }
    } else {
      free(buf);
      httpd_resp_send_404(req);
      return ESP_FAIL;
    }
    free(buf);
  }

  sprintf(resp, "OK");
  return httpd_resp_send(req, resp, strlen(resp));
}

void init_web() {
  httpd_config_t config = HTTPD_DEFAULT_CONFIG();

  httpd_uri_t index_uri = {
    .uri       = "/",
    .method    = HTTP_GET,
    .handler   = index_handler,
    .user_ctx  = NULL
  };

  httpd_uri_t shoot_uri = {
    .uri       = "/shoot",
    .method    = HTTP_GET,
    .handler   = shoot_handler,
    .user_ctx  = NULL
  };

  httpd_uri_t move_uri = {
    .uri       = "/move",
    .method    = HTTP_GET,
    .handler   = move_handler,
    .user_ctx  = NULL
  };

  httpd_uri_t homing_uri = {
    .uri       = "/homing",
    .method    = HTTP_GET,
    .handler   = homing_handler,
    .user_ctx  = NULL
  };

  httpd_uri_t set_uri = {
    .uri       = "/set",
    .method    = HTTP_GET,
    .handler   = set_handler,
    .user_ctx  = NULL
  };

  httpd_uri_t status_uri = {
    .uri       = "/status",
    .method    = HTTP_GET,
    .handler   = status_handler,
    .user_ctx  = NULL
  };

  if (httpd_start(&camera_httpd, &config) == ESP_OK) {
    httpd_register_uri_handler(camera_httpd, &index_uri);
    httpd_register_uri_handler(camera_httpd, &shoot_uri);
    httpd_register_uri_handler(camera_httpd, &homing_uri);
    httpd_register_uri_handler(camera_httpd, &move_uri);
    httpd_register_uri_handler(camera_httpd, &status_uri);
    httpd_register_uri_handler(camera_httpd, &set_uri);
  }
}
